from Products.Five  import BrowserView
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from zExceptions import Redirect

CONF_CLAMP_SIZE = 40

class SelectConferences(BrowserView):
    def __call__(self, *args, **kwargs):
        """
        Call the view
        """
        catalog = getToolByName(self.context, "portal_catalog")
        self.conferences = list(catalog(portal_type = "Talk",
                                        sort_on = "sortable_title"))
        self.get_checked()
        return super(SelectConferences, self).__call__(*args, **kwargs)

    def getAllConferences(self, only_checked=False):
        """
        Get all the conferences
        """

        # Group them by start date
        by_start = {}
        for conf in self.conferences:
            key = conf.start.strftime('%Y%m%d%H%M')
            if not key in by_start:
                by_start[key] = []
            by_start[key].append(conf)

        keys = by_start.keys()
        keys.sort()

        for key in keys:
            confs = by_start[key]
            vals = []
            for conf in confs:
                value = { 'UID': conf.UID,
                          'absolute_url': conf.getURL(),
                          'Title': conf.Title,
                          'Description': conf.Description,
                          'checked': self.checked[conf.UID] }
                if value['checked'] or not only_checked:
                    vals.append(value)
            if len(confs) > 1 and not only_checked:
                vals.insert(0, { 'UID': '',
                                 'absolute_url': None,
                                 'Title': 'None',
                                 'Description': "I don't want to attend any session at this time.",
                                 'checked': not [ c for c in vals if c['checked'] ] })
            step =  { 'start': conf.start,
                      'end': conf.end,
                      'type': len(confs) > 1 and 'radio' or 'checkbox',
                      'name': 'conf_%s' % key,
                      'confs': vals }
            yield step

    def get_checked(self):
        """
        Get where we are registered
        """
        member = getToolByName(self.context, "portal_membership").getAuthenticatedMember()
        self.email = member.getProperty('email')
        self.user = member.getId()
        self.checked = {}
        for conf in self.conferences:
            self.checked[conf.UID] = self.user in conf.getAttendees and 'checked' or None
        
        if "form.submitted" in self.request:
            self.update_registration()

    def update_registration(self):
        """
        Update where we are registered
        """
        selected = []
        for key,value in self.request.form.items():
            if key.startswith('conf_') and value:
                selected.append(value)

        public = self.request.form.get('ispublic', False) and True or False
        member = getToolByName(self.context, "portal_membership").getAuthenticatedMember()
        member.setMemberProperties({ 'is_public': public })
        
        ctool = getToolByName(self, "portal_cockpit_control")

        for conf in self.conferences:
            current = self.checked[conf.UID]
            wanted = (conf.UID in selected) and "checked" or None
            if wanted != current:
                self.checked[conf.UID] = wanted
                tagname = 'Conf ' + conf.Title
                tagname = tagname.replace(',', ' ')
                # If it's too long, just cut at a nearby space
                if len(tagname) > CONF_CLAMP_SIZE:
                    beg = tagname[:CONF_CLAMP_SIZE]
                    end = tagname[CONF_CLAMP_SIZE:]
                    space = end.find(' ')
                    if space != -1:                        
                        tagname = beg + end[:space]
                if wanted:
                    ctool.add_tag(email = self.email,
                                  tags = [ tagname ],
                                  rpcrule = 5)
                    new = conf.getAttendees + (self.user, )
                else:
                    ctool.del_tag(email = self.email,
                                  tags = [ tagname ],
                                  rpcrule = 5)
                    new = [ u for u in conf.getAttendees if u != self.user ]
                conf = conf.getObject()
                conf.setAttendees(tuple(new))
                conf.reindexObject()
            
        pu = getToolByName(self.context, "plone_utils")
        pu.addPortalMessage("Sessions you'll attend to saved.", 'info')
        
    def ensureProfileComplete(self):
        """
        Ensure our profile is complete, if not, redirect to profile
        """
        portal = getToolByName(self.context, "portal_url").getPortalObject()
        member = getToolByName(portal, "portal_membership").getAuthenticatedMember()
        fields = [ 'firstname', 'lastname', 'company', 'function', 'country' ]
        for field in fields:
            if not member.getProperty(field):
                pu = getToolByName(portal, "plone_utils")
                pu.addPortalMessage("Your profile is incomplete. Please complete it first.", 'error')
                raise Redirect, portal.absolute_url() + '/personalize_form'
