"""Define some gory object that will takes the precedence over the
default ``main_template`` and use it or the specific public template.

Why do we need that? Or, more precisely, why do I think we need that?

First, I tried to use a simple script called ``main_template`` that
returns the right... template. It throws an AttributeError on
``macros`` because templates use the macro with
``here/main_template/macros/master``. Since the script does not have
any ``macros`` attribute or method, we get an AttributeError. The
script itself is actually not called at all, it is only accessed.

I then tried to create a BrowserView. I can make a BrowserView
correctly respond to the call to a ``macros`` method or attribute. The
problem is that templates call ``here/main_template`` and Zope does
not look at browser views in this case. It would if the templates
called ``here/@@main_template`` (note the "@@" prefix). Too bad.

A third possibility is to create a template called ``main_template``.
Then, with a convolut^Wsmart usage of macros, it may be possible to
make it use the right template. I tried a few iterations of this and
grew impatient...

So I came up with the solution below. An object of this class
(``MainTemplateSelector``) is to be created at the root of the
portal. It will be named ``main_template``. When templates traverse
this object to a ``macros`` attribute or method, the object dutifully
returns the right template, depending on the request.
"""

from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem


def addMainTemplateSelector(context, REQUEST):
    container = context.Destination()
    container._setObject(MainTemplateSelector.id, MainTemplateSelector())
    redirect = REQUEST.RESPONSE.redirect
    return redirect('%s/manage_main' % container.absolute_url())


_default = object()

from Acquisition import Implicit

class AcquirableDict(dict, Implicit):
    """Yes, I know, that does not make sense. See below."""
    pass


class MainTemplateSelector(SimpleItem):
    """An object of this class is to be created at the root of the
    Plone portal. See module docstring above for further details.
    """

    meta_type = 'MainTemplateSelector'
    id = 'main_template'

    def __bobo_traverse__(self, request, name):
        if name == 'macros':
            ## Yes, we use self.REQUEST and not request because it is
            ## not the same. The latter is actually quite incomplete.
            if 'admin' in self.REQUEST.get('HTTP_X_FORWARDED_HOST',
                                           self.REQUEST['HTTP_HOST']):
                template_path = '@@main_template'
            else:
                template_path = '@@public_template'
            template = self.unrestrictedTraverse(template_path)
            ## For some reasons, the traversal machinery would like
            ## very much the value to be acquired so that 'validate()'
            ## can fetch the required roles from somewhere. Being nice
            ## and diligent (and also half-awake at 3 A.M.), I humbly
            ## comply.
            return AcquirableDict(template['macros']).__of__(self)
        return getattr(self, name)

InitializeClass(MainTemplateSelector)
