from AccessControl import ClassSecurityInfo
from zope.interface import implements

try:
    from Products.LinguaPlone import public as atapi
except ImportError:
    from Products.Archetypes import atapi
from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.content.document import ATDocumentBase
from Products.ATContentTypes.content.image import ATCTImageTransform
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from iw.fss.FileSystemStorage import FileSystemStorage

from pilot.conference import config
from pilot.conference import MSG_FACTORY as _
from pilot.conference.interfaces import ISpeaker


SCHEMA = ATContentTypeSchema.copy() + atapi.Schema((
        atapi.ComputedField(
            'title',
            searchable=1,
            expression='context.getFullname()',
            accessor='Title',
            ),

        atapi.StringField(
            'firstName',
            searchable=True,
            languageIndependent=True,
            widget=atapi.StringWidget(label=_('First name')),
            ),

        atapi.StringField(
            'lastName',
            searchable=True,
            languageIndependent=True,
            widget=atapi.StringWidget(label=_('Last name')),
            ),

        atapi.ImageField(
            'picture',
            languageIndependent=True,
            sizes=dict(thumb=(200, 200)),
            widget=atapi.ImageWidget(label=_('Picture')),
            storage=FileSystemStorage(),
            ),

        atapi.TextField(
            'text',
            searchable=True,
            primary=True,
            storage=atapi.AnnotationStorage(migrate=True),
            validators=('isTidyHtmlWithCleanup', ),
            default_output_type='text/x-html-safe',
            widget=atapi.RichWidget(
                label=_(u'Biography'),
                rows=25,
                allow_file_upload=zconf.ATDocument.allow_document_upload)
            ),

        atapi.StringField(
            'homePage',
            languageIndependent=True,
            widget=atapi.StringWidget(label=_('Home page')),
            ),
        ))
SCHEMA['description'].widget.label = _('Position and company/organization')
SCHEMA['description'].widget.description = ''
SCHEMA.moveField('description', after='lastName')
finalizeATCTSchema(SCHEMA)


class Speaker(ATDocumentBase, ATCTImageTransform):
    """A speaker."""

    implements(ISpeaker)

    schema = SCHEMA

    security = ClassSecurityInfo()
    decProtected = security.declareProtected


    decProtected('View', 'getFullname')
    def getFullname(self):
        """Return speaker's full name."""
        return u' '.join((self.getFirstName(), self.getLastName()))


    decProtected('View', 'tag')
    def tag(self, **kwargs):
        """Generate image tag using the API of the ImageField.

        This has been taken from ``ATNewsItem``.
        """
        if 'title' not in kwargs:
            kwargs['title'] = self.Title()
        return self.getField('picture').tag(self, **kwargs)


    def __bobo_traverse__(self, REQUEST, name):
        """Transparent access to picture scales.

        This has been taken from ``ATNewsItem``.
        """
        if name.startswith('picture'):
            field = self.getField('picture')
            image = None
            if name == 'picture':
                image = field.getScale(self)
            else:
                scalename = name[len('picture_'):]
                if scalename in field.getAvailableSizes(self):
                    image = field.getScale(self, scale=scalename)
            if image is not None and not isinstance(image, basestring):
                # image might be None or '' for empty images
                return image

        return ATDocumentBase.__bobo_traverse__(self, REQUEST, name)

atapi.registerType(Speaker, config.PROJECTNAME)
