from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from zope.interface import implements

try:
    from Products.LinguaPlone import public as atapi
except ImportError:
    from Products.Archetypes import atapi
from Products.ATContentTypes import ATCTMessageFactory as _atct_
from Products.ATContentTypes.configuration import zconf
from Products.ATContentTypes.content.folder import ATFolder
from Products.ATContentTypes.content.folder import ATFolderSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema
from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget import ReferenceBrowserWidget
from Products.CMFCore.utils import getToolByName

from pilot.conference import config
from pilot.conference import MSG_FACTORY as _
from pilot.conference.interfaces import ITalk


SCHEMA = ATFolderSchema.copy() + atapi.Schema((
       atapi.StringField(
               'eventlocation',
               required=True,
               searchable=False,
               #accessor='start',
               #default_method=DateTime,
               languageIndependent=True,
               widget=atapi.StringWidget(
               label=_atct_(u'label_event_location', default=u'Event Location')),
                          ),
   
    
        atapi.DateTimeField(
            'startDate',
            required=True,
            searchable=False,
            accessor='start',
            default_method=DateTime,
            languageIndependent=True,
            widget=atapi.CalendarWidget(
                label=_atct_(u'label_event_start', default=u'Event Starts')),
            ),

        atapi.DateTimeField(
            'endDate',
            required=True,
            searchable=False,
            accessor='end',
            default_method=DateTime,
            languageIndependent=True,
            widget=atapi.CalendarWidget(
                label=_atct_(u'label_event_end', default=u'Event Ends')),
            ),

        atapi.TextField(
            'text',
            required=False,
            searchable=True,
            primary=True,
            storage=atapi.AnnotationStorage(migrate=True),
            validators=('isTidyHtmlWithCleanup', ),
            default_output_type='text/x-html-safe',
            widget=atapi.RichWidget(
                label=_(u'Text'),
                rows=25,
                allow_file_upload=zconf.ATDocument.allow_document_upload)
            ),

        #atapi.ReferenceField(
        #    'hosts',
        #    required=False,
        #    relationship='hostedBy',
        #    multiValued=True,
        #    languageIndependent=True,
        #    index='KeywordIndex', ## FIXME
        #    widget=ReferenceBrowserWidget(
        #        startup_directory='/attend/speakers',
        #        allow_sorting=True,
        #        label=_(u'Hosts'),
        #        ),
        #    ),

        atapi.ReferenceField(
            'speakers',
            required=False,
            relationship='withSpeakers',
            multiValued=True,
            languageIndependent=True,
            index='KeywordIndex', ## FIXME
            widget=ReferenceBrowserWidget(
                startup_directory='/attend/speakers',
                allow_sorting=True,
                label=_(u'Speakers'),
                ),
            ),

        atapi.ReferenceField(
            'organizers',
            required=False,
            relationship='withOrganizers',
            multiValued=True,
            languageIndependent=True,
            index='KeywordIndex', ## FIXME
            widget=ReferenceBrowserWidget(
                startup_directory='/attend/speakers',
                allow_sorting=True,
                label=_(u'Organizers'),
                ),
            ),

        atapi.ReferenceField(
            'moderators',
            required=False,
            relationship='withModerators',
            multiValued=True,
            languageIndependent=True,
            index='KeywordIndex', ## FIXME
            widget=ReferenceBrowserWidget(
                startup_directory='/attend/speakers',
                allow_sorting=True,
                label=_(u'Moderators'),
                ),
            ),

        atapi.BooleanField(
            'hideAttendees',
            required=False,
            languageIndependent=True,
            widget=atapi.BooleanWidget(
                label=_(u'Hide attendees list'),
            ),
        ),
       
        atapi.LinesField(
            'attendees',
            required=False,
            multiValued=True,
            languageIndependent=True,
            visible = {'edit':'invisible', 'view':'invisible'},
            index='KeywordIndex:schema',
            widget=atapi.LinesWidget(),
            ),
))
finalizeATCTSchema(SCHEMA)


class Talk(ATFolder):
    """A directory."""

    implements(ITalk)

    schema = SCHEMA

    security = ClassSecurityInfo()
    decProtected = security.declareProtected


    decProtected('View', 'getFiles')
    def getFiles(self):
        """Return attached files."""
        cat = getToolByName(self, 'portal_catalog')
        return cat(path=self.getPhysicalPath(), meta_type='ATFile')

    decProtected('View', 'getFiles')
    def getSpeakers(self):
        """Return sorted Speakers."""
        speakers_sort=self.schema["speakers"].get(self)
        speakers_sort.sort(lambda e1,e2: cmp(e1.getLastName(), e2.getLastName()))
        return speakers_sort

    decProtected('View', 'getFiles')
    def getOrganizers(self):
        """Return sorted Organizers."""
        organizers_sort=self.schema["organizers"].get(self)
        organizers_sort.sort(lambda e1,e2: cmp(e1.getLastName(), e2.getLastName()))
        return organizers_sort

    decProtected('View', 'getFiles')
    def getModerators(self):
        """Return sorted Organizers."""
        moderators_sort=self.schema["moderators"].get(self)
        moderators_sort.sort(lambda e1,e2: cmp(e1.getLastName(), e2.getLastName()))
        return moderators_sort
        
    decProtected('View', 'getFilteredAttendees')
    def getFilteredAttendees(self):
        """
        Get the list of attendees we are allowed to see
        """
        pm = getToolByName(self, 'portal_membership')
        member = pm.getAuthenticatedMember()
        manager = member.hasRole(self, [ 'Manager' ])
        hide  = self.getHideAttendees()
        if hide and not manager:
            return []
        res = []
        for attendee in self.getAttendees():
            attendee = pm.getMemberById(attendee)
            if not attendee:
                continue
            if manager or attendee.getProperty('is_public', True):
                res.append(attendee)
        return res
        

atapi.registerType(Talk, config.PROJECTNAME)
